package ui

import (
	"fmt"
	"html/template"
	"log"
	"net/http"
	"strconv"

	"streifling.com/jason/cpolis/cmd/data"
)

type AddUserData struct {
	data.User
	Msg string
}

func inputsEmpty(user data.User, pass, pass2 string) bool {
	return len(user.UserName) == 0 ||
		len(user.FirstName) == 0 ||
		len(user.LastName) == 0 ||
		len(pass) == 0 ||
		len(pass2) == 0
}

func checkUserStrings(user data.User) (string, int, bool) {
	userLen := 15
	nameLen := 50

	if len(user.UserName) > userLen {
		return "Benutzername", userLen, false
	} else if len(user.FirstName) > nameLen {
		return "Vorname", nameLen, false
	} else if len(user.LastName) > nameLen {
		return "Nachname", nameLen, false
	} else {
		return "", 0, true
	}
}

func HomePage(db *data.DB) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		numRows, err := db.CountEntries()
		if err != nil {
			log.Fatalln(err)
		}

		if numRows == 0 {
			template.Must(template.ParseFiles("web/templates/index.html", "web/templates/add-user.html")).Execute(w, nil)
		} else {
			template.Must(template.ParseFiles("web/templates/index.html", "web/templates/login.html")).Execute(w, nil)
		}
	}
}

func CreateUser() http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		template.Must(template.ParseFiles("web/templates/add-user.html")).ExecuteTemplate(w, "page-content", nil)
	}
}

func AddUser(db *data.DB) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		role, err := strconv.Atoi(r.PostFormValue("role"))
		if err != nil {
			log.Println(err)
			http.Error(w, err.Error(), http.StatusInternalServerError)
			return
		}

		htmlData := AddUserData{
			User: data.User{
				UserName:  r.PostFormValue("username"),
				FirstName: r.PostFormValue("first-name"),
				LastName:  r.PostFormValue("last-name"),
				Role:      data.Role(role),
			},
		}
		pass := r.PostFormValue("password")
		pass2 := r.PostFormValue("password2")

		if inputsEmpty(htmlData.User, pass, pass2) {
			htmlData.Msg = "Alle Felder müssen ausgefüllt werden."
			template.Must(template.ParseFiles("web/templates/add-user.html")).ExecuteTemplate(w, "page-content", htmlData)
			return
		}
		userString, stringLen, ok := checkUserStrings(htmlData.User)
		if !ok {
			htmlData.Msg = fmt.Sprint(userString, " ist zu lang. Maximal ", stringLen, " Zeichen erlaubt.")
			template.Must(template.ParseFiles("web/templates/add-user.html")).ExecuteTemplate(w, "page-content", htmlData)
			return
		}
		id, _ := db.GetID(htmlData.UserName)
		if id != 0 {
			htmlData.Msg = fmt.Sprint(htmlData.UserName, " ist bereits vergeben. Bitte anderen Benutzernamen wählen.")
			template.Must(template.ParseFiles("web/templates/add-user.html")).ExecuteTemplate(w, "page-content", htmlData)
			return
		}
		if pass != pass2 {
			htmlData.Msg = "Die Passwörter stimmen nicht überein."
			template.Must(template.ParseFiles("web/templates/add-user.html")).ExecuteTemplate(w, "page-content", htmlData)
			return
		}

		if err := db.AddUser(htmlData.User, pass); err != nil {
			log.Println(err)
			http.Error(w, err.Error(), http.StatusInternalServerError)
			return
		}
		template.Must(template.ParseFiles("web/templates/hub.html")).ExecuteTemplate(w, "page-content", nil)
	}
}
