package main

import (
	"log"
	"net/http"
	"os"

	"streifling.com/jason/cpolis/cmd/data"
	"streifling.com/jason/cpolis/cmd/ui"
)

func main() {
	logFile, err := os.Create("tmp/cpolis.log")
	if err != nil {
		log.Fatalln(err)
	}
	defer logFile.Close()
	log.SetOutput(logFile)

	db, err := data.OpenDB("cpolis")
	if err != nil {
		log.Fatalln(err)
	}
	defer db.Close()

	feed, err := data.OpenFeed("tmp/rss.gob")
	if err != nil {
		log.Println(err)
		feed = data.NewFeed("Freimaurer Distrikt Niedersachsen und Sachsen-Anhalt",
			"https://distrikt-ni-st.de",
			"Freiheit, Gleichheit, Brüderlichkeit, Toleranz und Humanität")
	}

	articleList := data.NewArticleList()

	mux := http.NewServeMux()
	mux.Handle("/web/static/", http.StripPrefix("/web/static/", http.FileServer(http.Dir("web/static/"))))

	mux.HandleFunc("/", ui.HomePage(db))
	mux.HandleFunc("/rss/", ui.ShowRSS(feed))

	mux.HandleFunc("GET /hub/", ui.ShowHub())

	mux.HandleFunc("POST /add-user/", ui.AddUser(db))
	mux.HandleFunc("POST /create-user/", ui.CreateUser())
	mux.HandleFunc("POST /finish-article/", ui.FinishArticle(articleList))
	mux.HandleFunc("POST /login/", ui.Login(db))
	mux.HandleFunc("POST /review-article/", ui.ReviewArticle(articleList))
	mux.HandleFunc("POST /publish-article/", ui.PublishArticle(feed, articleList))
	mux.HandleFunc("POST /unpublished-articles/", ui.ShowUnpublishedArticles(articleList))
	mux.HandleFunc("POST /write-article/", ui.WriteArticle())

	log.Fatalln(http.ListenAndServe(":8080", mux))
}
